##  Author: Stefan Evert
## Purpose: Sample participant for free association task - The FOO system (first-order only)

## load database of first-order associations (may take a while)
DB <- read.delim(gzfile("lexsem08_first_order_associations.ds.gz"))

## Task 1: load training and test data
Train <- read.delim("FA/discrimination_train.tbl")
Test <- read.delim("FA/discrimination_test.tbl")

## annotate training data with association scores from DB
Train.annot <- merge(Train, DB, by.x=c("cue","target"), by.y=c("l1","l2"), all.x=TRUE)
Train.annot <- subset(Train.annot, select=c("cue", "target", "type", "am.log.likelihood", "am.t.score", "am.MI", "am.Dice"))

## now calculate decision accuracy vectors for each association measure and threshold
TP <- ifelse(Train.annot$type == "FIRST", TRUE, FALSE) # just distinguish "FIRST" from "HAPAX"/"RANDOM" (TP = true positives)
n.TP <- sum(TP) # number of true associations in training set
for (m in c("am.log.likelihood", "am.t.score", "am.MI", "am.Dice")) {
  cat("Association measure:", m, "\n")
  scores <- Train.annot[[m]]
  scores[is.na(scores)] <- 0 # missing values -> word pair is not in database
  ranking <- order(scores, decreasing=TRUE)
  n <- length(ranking)
  ## for each position j in 1:n, we have cumsum(...)[j] true positives, and (n - n.TP) - (j - cumsum(...)[j]) true negatives
  ## => accuracy = (2 * cumsum(...)[j] - j + n - n.TP) / n
  accuracy <- 100 * (2 * cumsum(TP[ranking]) - (1:n) + n - n.TP) / n # in percent
  print(round(accuracy,2))
  best.idx <- which.max(accuracy)
  cat("  best threshold is >= ", scores[ranking[best.idx]], " with accuracy of ", round(accuracy[best.idx],2), "%\n")
}
## => best measure is t-score with threshold of 3.405

## use best model from training (t-score >= 3.405) for classification of test set
Test.annot <- merge(Test, DB, by.x=c("cue","target"), by.y=c("l1","l2"), all.x=TRUE)
scores <- Test.annot$am.t.score
scores[is.na(scores)] <- 0
foo.task1 <- data.frame(cue=Test.annot$cue, target=Test.annot$target, model.class=ifelse(scores >= 3.405, "FIRST", "OTHER"))
write.table(foo.task1, "foo_output/task1.tbl", sep="\t", row.names=FALSE, col.names=TRUE, quote=FALSE)


## Task 2: load training and test data
Train <- read.delim("FA/correlation_train.tbl")
Test <- read.delim("FA/correlation_test.tbl")

## annotate training data with association scores from DB (as above)
Train.annot <- merge(Train, DB, by.x=c("cue","target"), by.y=c("l1","l2"), all.x=TRUE)
Train.annot <- subset(Train.annot, select=c("cue", "target", "assoc", "am.log.likelihood", "am.t.score", "am.MI", "am.Dice"))
Train.annot <- na.omit(Train.annot)       # can't do sensible correlations with missing values

## use scatterplots to explore correlation between association measures and free association ratings 
plot(assoc ~ am.log.likelihood, data=Train.annot) # outliers ruin linear correlation
plot(assoc ~ log10(am.log.likelihood), data=Train.annot) # log-scaled association scores look much better
plot(assoc ~ log10(am.t.score), data=Train.annot)        # very weak correlation
plot(assoc ~ log10(am.MI), data=Train.annot)             # looks relatively good with steep regression line (except for a few outliers)
abline(lm(assoc ~ log10(am.MI), data=Train.annot), lwd=2, col="blue")
print(cor.test(~ assoc + log10(am.MI), data=Train.annot))
plot(assoc ~ log10(am.Dice), data=Train.annot)           # also relatively good, less steep regression and slightly better correlation
abline(lm(assoc ~ log10(am.Dice), data=Train.annot), lwd=2, col="blue")
print(cor.test(~ assoc + log10(am.Dice), data=Train.annot))

## intuitively, MI seems to have best correlation if outliers (assoc > 2 * log10(MI) + .5) are omitted
Ta2 <- subset(Train.annot, assoc < 2 * log10(am.MI) + 0.5)
plot(assoc ~ log10(am.MI), data=Ta2)             # looks relatively good with steep regression line (except for a few outliers)
regr <- lm(assoc ~ log10(am.MI), data=Ta2)
abline(regr, lwd=2, col="blue")
print(cor.test(~ assoc + log10(am.MI), data=Ta2)) # correlation coefficient is up to 0.64 (from 0.46)
print(regr)
## => best regression model: predicted assoc = .75 * log10(MI) + .2

## use this model to predict free associations on test set
Test.annot <- merge(Test, DB, by.x=c("cue","target"), by.y=c("l1","l2"), all.x=TRUE)
scores <- Test.annot$am.MI
predict.assoc <- .75 * log10(scores) + .2
predict.assoc[is.na(predict.assoc)] <- 0 # cue-target pair not in database => predict zero association
foo.task2 <- data.frame(cue=Test.annot$cue, target=Test.annot$target, predict.assoc=predict.assoc)
write.table(foo.task2, "foo_output/task2.tbl", sep="\t", row.names=FALSE, col.names=TRUE, quote=FALSE)
